<?php
/*
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License v2
 as published by the Free Software Foundation.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


 Connects to the scrobbler servers and submits "Now Playing" & "Track Summissions" 
  - Last.fm Submissions Protocol v2
*/

class scrobbler {
	public $queued_tracks;
	public $error_msg;
	public $socket_timeout;
	private $api_key, $secret, $session_key;

	public function __construct($session_key = null) {
		$this->queued_tracks = array();

		$this->error_msg = '';
		$this->api_key = ZINA_LASTFM_API_KEY;
		$this->secret = ZINA_LASTFM_SECRET;
		$this->socket_timeout = 2;
		$this->session_key = $session_key;
	}

	public function getToken() {
		return 	'http://www.last.fm/api/auth/?api_key='.$this->api_key.'&cb='.zurl('','l=80',NULL, TRUE);
	}

	public function getSig($vars) {
		ksort($vars);

		$sig = '';
		foreach($vars as $name => $value) {
			$sig .= $name.$value;
		}
		$sig .= $this->secret;
		$sig = md5($sig);

		return $sig;
	}

	public function getSession($token) {
		$vars = array(
			'api_key' => $this->api_key,
			'method' => 'auth.getSession',
			'token' => $token
		);
		$vars['api_sig'] = $this->getSig($vars);

		if ($response = $this->apiGetCall($vars)) {
			return (string) $response->session->key;
		} else {
			return FALSE;
		}
	}

	public function getAlbumInfo($track) {
		$vars = array(
			'api_key' => $this->api_key,
			'method' => 'album.getInfo'
		);
		$vars = array_merge($vars, $track);
		
		$info = array();
		if ($call = $this->apiGetCall($vars)) {
			$info['name'] = (string) $call->album->name;
			$info['artist'] = (string) $call->album->artist;
			$info['lastfmid'] = (string) $call->album->id;
			$info['mbid'] = (string) $call->album->mbid;
			$info['url'] = (string) $call->album->url;
			$info['releasedate'] = strtotime(trim((string) $call->album->releasedate));
			$info['image']['small'] = (string) $call->album->image;
			$info['image']['medium'] = (string) $call->album->image[1];
			$info['image']['large'] = (string) $call->album->image[2];
			$info['image']['extralarge'] = (string) $call->album->image[3];
			$info['image']['mega'] = (string) $call->album->image[4];
			$info['listeners'] = (string) $call->album->listeners;
			$info['playcount'] = (string) $call->album->playcount;
			$info['summary'] = (string) $call->album->wiki->summary;
			$info['content'] = (string) $call->album->wiki->content;
			
			return $info;
		} else {
			return FALSE;
		}
	}
	public function getArtistInfo($track) {
		$vars = array(
			'api_key' => $this->api_key,
			'method' => 'artist.getInfo'
		);
		$vars = array_merge($vars, $track);
		
		$info = array();
		if ($call = $this->apiGetCall($vars)) {
			$info['image']['small'] = (string) $call->artist->image;
			$info['image']['medium'] = (string) $call->artist->image[1];
			$info['image']['large'] = (string) $call->artist->image[2];
			$info['image']['extralarge'] = (string) $call->artist->image[3];
			$info['image']['mega'] = (string) $call->artist->image[4];
			$info['summary'] = (string) $call->artist->bio->summary;
			$info['content'] = (string) $call->artist->bio->content;
			
			return $info;
		} else {
			return FALSE;
		}
	}	

	public function updateNowPlaying($track) {
		if(empty($track) || !is_object($track)) {
			$this->error_msg = "No track to submit to now playing";
			return false;
		}

		if (empty($this->session_key)) {
			$this->error_msg = "No session key";
			return false;
		}

		$vars = array(
			'api_key' => $this->api_key,
			'method' => 'track.updateNowPlaying',
			'sk' => $this->session_key,
			'artist' => $track->artist,
			'track' => $track->title,
			'album' => $track->album,
			'trackNumber' => $track->track,
			'duration' => $track->length
		);
		$vars['api_sig'] = $this->getSig($vars);

		if ($response = $this->apiPostCall($vars)) {
			return true;
			#return (string) $response->nowplaying->ignoredMessage;
		} else {
			return false;
		}
	}


	public function trackLove($track, $love = true) {
		if(empty($track) || !is_object($track)) {
			$this->error_msg = "No track to submit to now playing";
			return false;
		}

		if (empty($this->session_key)) {
			$this->error_msg = "No session key";
			return false;
		}

		$vars = array(
			'api_key' => $this->api_key,
			'method' => ($love) ? 'track.love' : 'track.unlove',
			'sk' => $this->session_key,
			'artist' => $track->artist,
			'track' => $track->title,
		);
		$vars['api_sig'] = $this->getSig($vars);

		if ($response = $this->apiPostCall($vars)) {
			return true;
			#return (string) $response->nowplaying->ignoredMessage;
		} else {
			$this->error_msg = "Post Call Failed";
			return false;
		}
	}

	private function apiGetCall($vars) {
		$socket = fsockopen('ws.audioscrobbler.com', 80, $errno, $errstr, $this->socket_timeout);
		if(!$socket) {
			$this->error_msg = 'lastfm->apiGetCall->fsockopen failed: '.$errstr;
			return false;
		}

		$url = '/2.0/?';
		foreach ($vars as $name => $value) {
			$url .= trim(urlencode($name)).'='.trim(urlencode($value)).'&';
		}
		$url = substr($url, 0, -1);
		$url = str_replace(' ', '%20', $url);

		stream_set_timeout($socket, $this->socket_timeout);
		fwrite($socket, "GET $url HTTP/1.1\r\n");
		fwrite($socket, "Host: ws.audioscrobbler.com\r\n");
		fwrite($socket, "Accept: */*\r\n\r\n");

		$response = '';
		while(!feof($socket)) {
			$response .= fgets($socket, 4096);
		}
		fclose($socket);

		$response = preg_replace('/^.*?</s', '<', $response);

		libxml_use_internal_errors(true);
		$xml = simplexml_load_string($response);
		return $xml;
	}

	private function apiPostCall($vars) {
		$socket = fsockopen('ws.audioscrobbler.com', 80, $errno, $errstr, $this->socket_timeout);
		if(!$socket) {
			$this->error_msg = 'lastfm->apiPostCall->fsockopen failed: '.$errstr;
			return false;
		}

		$data = '';
		foreach ($vars as $name => $value) {
			$data .= trim(urlencode($name)).'='.trim(urlencode($value)).'&';
		}
		$data = substr($data, 0, -1);

		stream_set_timeout($socket, $this->socket_timeout);
		
		$out = "POST /2.0/ HTTP/1.1\r\n".
			"Host: ws.audioscrobbler.com\r\n".
			#"User-Agent: Zina/2.0\r\n".
			"Content-Length: ".strlen($data)."\r\n".
			"Content-Type: application/x-www-form-urlencoded\r\n\r\n".
			$data."\r\n";

		fwrite($socket, $out);

		$response = '';
		while(!feof($socket)) {
			$response .= fgets($socket, 4096);
		}
		fclose($socket);

		$response = preg_replace('/^.*?</s', '<', $response);
		#zina_debug($response);

		#TODO: better error crap
		libxml_use_internal_errors(true);
		$xml = simplexml_load_string($response);
		return $xml;
	}

	public function get_queue_count() {
		return count($this->queued_tracks);
	}

	/**
	 * queue_track 
	 * This queues the LastFM track by storing it in this object
	 * $track is object containing 'artist', 'album', 'title','track','length', 'timestamp'
	 */
	public function queue_track($track) {
		if(empty($track) || !is_object($track)) {
			$this->error_msg = "No track to submit to queue";
		} elseif ($track->length < 30 || time() - $track->timestamp < 30) {
			$this->error_msg = "Track too short, not queing";
		} elseif (time() - $track->timestamp < $track->length/2) {
			$this->error_msg = "Track not played long enough, not queing";
		} else {
			$this->queued_tracks[$track->timestamp] = $track;
			return true;
		}	
		return false;
	}

	public function submit_track($track) {
		if (!empty($this->session_key) && $this->queue_track($track)) {
			return $this->submit_tracks();
		}
		return false;
	}

	public function submit_tracks() {

		if (!count($this->queued_tracks)) {
			$this->error_msg = "No tracks to submit";
			return false;
		}

		//sort array by timestamp
		ksort($this->queued_tracks); 

		$vars = array(
			'api_key' => $this->api_key,
			'method' => 'track.scrobble',
			'sk' => $this->session_key
		);

		$i = 0;

		foreach ($this->queued_tracks as $track) {
			$vars["artist[$i]"] = $track->artist;
			$vars["track[$i]"] = $track->title;
			$vars["album[$i]"] = $track->album;
			$vars["duration[$i]"] = $track->length;
			$vars["timestamp[$i]"] = $track->timestamp;
			$vars["trackNumber[$i]"] = $track->track;
			$i++;
		}
		$vars['api_sig'] = $this->getSig($vars);

		if ($response = $this->apiPostCall($vars)) {
			return true;
			#return (string) $response->nowplaying->ignoredMessage;
		} else {
			return false;
		}
	}
}
?>
